package org.jeecg.modules.adweb.seo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xkcoding.http.util.StringUtil;

import jakarta.annotation.Resource;

import lombok.extern.slf4j.Slf4j;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.jeecg.common.util.FastJsonUtil;
import org.jeecg.modules.adweb.common.util.DateUtil;
import org.jeecg.modules.adweb.common.util.ListUtil;
import org.jeecg.modules.adweb.seo.entity.*;
import org.jeecg.modules.adweb.seo.mapper.SeoKeywordsMapper;
import org.jeecg.modules.adweb.seo.mapper.SeoPlanSubscriptionMapper;
import org.jeecg.modules.adweb.seo.service.*;
import org.jeecg.modules.adweb.seo.vo.RankInfoVO;
import org.jeecg.modules.adweb.seo.vo.SeoRankInfoVO;
import org.jeecg.modules.adweb.seo.vo.ServerTimeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Zenas
 */
@Slf4j
@Service
public class SearchKeywordsServiceImpl implements ISearchKeywordsService {

    @Autowired private ISeoKpiStatisticsService seoKpiStatisticsService;

    @Resource private SeoPlanSubscriptionMapper seoPlanSubscriptionMapper;

    @Resource private SeoKeywordsMapper seoKeywordsMapper;

    @Autowired private ISeoKeywordsService seoKeywordsService;

    @Autowired private ISeoKeywordsSerpService seoKeywordsSerpService;

    @Autowired private ISeoPlanSubscriptionService seoPlanSubscriptionService;

    @Autowired private IHistoryReachStandardSiteService historyReachStandardSiteService;

    /**
     * 获得综合统计的数据
     *
     * @param siteCode 站点code
     * @param historyId 营销方案id
     * @return 综合统计的数据
     */
    @Override
    public ComprehensiveStatistics getComprehensiveInfo(String siteCode, String historyId) {
        ComprehensiveStatistics comprehensiveStatistics = new ComprehensiveStatistics();

        // 查询指标
        QueryWrapper<SeoKpiStatistics> statisticsQueryWrapper = new QueryWrapper<>();
        statisticsQueryWrapper.eq("host_code", siteCode);
        statisticsQueryWrapper.eq("del_flag", 0);
        statisticsQueryWrapper.eq("type", 1);
        statisticsQueryWrapper.eq("history_id", historyId);
        List<SeoKpiStatistics> seoKpiStatisticsList =
                seoKpiStatisticsService.list(statisticsQueryWrapper);
        if (ListUtil.isEmpty(seoKpiStatisticsList)) {
            return comprehensiveStatistics;
        }

        // 查询每个指标对应的完成数据
        for (SeoKpiStatistics kpiStatistics : seoKpiStatisticsList) {
            if (kpiStatistics.getKpiTarget() == null || kpiStatistics.getKpiTarget() == 0) {
                continue;
            }
            if ("APPOINT_KEYWORDS".equals(kpiStatistics.getKpiCode())) {
                Integer standard = seoPlanSubscriptionMapper.getKeywordStandard(siteCode);
                String datesStr = "";
                List<String> dateList =
                        seoKeywordsService.getKeywordsLastSevenDays(siteCode, 1, historyId);
                if (ListUtil.isEmpty(dateList)) {
                    comprehensiveStatistics.setAppointKeywordNum(
                            0 + " / " + kpiStatistics.getKpiTarget());
                    continue;
                } else {
                    for (int i = 0; i < dateList.size(); i++) {
                        if (i == dateList.size() - 1) {
                            datesStr += "'" + dateList.get(i) + "'";
                        } else {
                            datesStr += "'" + dateList.get(i) + "'" + ",";
                        }
                    }
                    datesStr = "(" + datesStr + ")";
                }
                RankInfoVO rankInfo =
                        seoKeywordsService.getSeoRankInfo(
                                siteCode, historyId, 1, standard, datesStr, "1");
                if (rankInfo != null) {
                    comprehensiveStatistics.setAppointKeywordNum(
                            rankInfo.getKeywordNum() + " / " + kpiStatistics.getKpiTarget());
                } else {
                    comprehensiveStatistics.setAppointKeywordNum(
                            0 + " / " + kpiStatistics.getKpiTarget());
                }
            } else if ("LONG_TAIL_KEYWORDS".equals(kpiStatistics.getKpiCode())) {
                String datesStr = "";
                List<String> dateList =
                        seoKeywordsService.getKeywordsLastSevenDays(siteCode, 2, historyId);
                if (ListUtil.isEmpty(dateList)) {
                    comprehensiveStatistics.setLongTailKeywordNum(
                            0 + " / " + kpiStatistics.getKpiTarget());
                    continue;
                } else {
                    for (int i = 0; i < dateList.size(); i++) {
                        if (i == dateList.size() - 1) {
                            datesStr += "'" + dateList.get(i) + "'";
                        } else {
                            datesStr += "'" + dateList.get(i) + "'" + ",";
                        }
                    }
                    datesStr = "(" + datesStr + ")";
                }
                RankInfoVO rankInfo =
                        seoKeywordsService.getSeoRankInfo(
                                siteCode, historyId, 1, 10, datesStr, "2");
                if (rankInfo != null) {
                    comprehensiveStatistics.setLongTailKeywordNum(
                            rankInfo.getKeywordNum() + " / " + kpiStatistics.getKpiTarget());
                } else {
                    comprehensiveStatistics.setLongTailKeywordNum(
                            0 + " / " + kpiStatistics.getKpiTarget());
                }
            } else if ("OUTER_CHAIN".equals(kpiStatistics.getKpiCode())) {
                comprehensiveStatistics.setOuterLinkNum(
                        kpiStatistics.getFinishTarget() + " / " + kpiStatistics.getKpiTarget());
            } else if ("ARTICLE_INSITE".equals(kpiStatistics.getKpiCode())) {
                log.info(
                        "getComprehenInfo ---- ARTICLE_INSITE - kpiStatistics:{}",
                        FastJsonUtil.toJSONString(kpiStatistics));

                // 为了保持数据一致，只走定时器获取数据库中的数据 2023-09-08
                comprehensiveStatistics.setArticleNum(
                        kpiStatistics.getFinishTarget() + " / " + kpiStatistics.getKpiTarget());
            }
        }
        return comprehensiveStatistics;
    }

    @Override
    public Map getRankInfo(String siteCode, String historyId) throws ParseException {
        List<SeoRankInfoVO> seoRankInfos = this.getSeoRankInfo(siteCode, historyId);
        ServerTimeVO ServerTimeVO = this.getServerTime(siteCode, historyId);

        Map map = new HashMap<>();
        map.put("appointKeyword", seoRankInfos.get(0));
        map.put("longTailKeyword", seoRankInfos.get(1));
        map.put("ServerTime", ServerTimeVO);
        return map;
    }

    /**
     * 获取一个站点的关键词排名数量
     *
     * @param siteCode 站点code
     * @return 关键词排名数量
     */
    @Override
    public List<SeoRankInfoVO> getSeoRankInfo(String siteCode, String historyId)
            throws ParseException {
        // 排名信息
        SeoRankInfoVO appointRankInfo = new SeoRankInfoVO();
        SeoRankInfoVO longTailRankInfo = new SeoRankInfoVO();
        // 1-10
        Map<String, Integer> first = dealKeywordsRankData(siteCode, historyId, 1, 10);
        if (first.containsKey("appoint")) {
            appointRankInfo.setFirstNum(first.get("appoint"));
        }
        if (first.containsKey("longTail")) {
            longTailRankInfo.setFirstNum(first.get("longTail"));
        }

        // 11-30
        Map<String, Integer> second = dealKeywordsRankData(siteCode, historyId, 11, 30);
        if (second.containsKey("appoint")) {
            appointRankInfo.setSecondNum(second.get("appoint"));
        }
        if (second.containsKey("longTail")) {
            longTailRankInfo.setSecondNum(second.get("longTail"));
        }
        // 31-100
        Map<String, Integer> third = dealKeywordsRankData(siteCode, historyId, 31, 100);
        if (third.containsKey("appoint")) {
            appointRankInfo.setThirdType(third.get("appoint"));
        }
        if (third.containsKey("longTail")) {
            longTailRankInfo.setThirdType(third.get("longTail"));
        }
        List<SeoRankInfoVO> list = new ArrayList<>();
        list.add(appointRankInfo);
        list.add(longTailRankInfo);
        return list;
    }

    /**
     * 获取服务时间
     *
     * @param siteCode 站点code
     * @param historyId 营销方案id
     * @return 服务情况，达标时间以及剩余时间
     */
    private ServerTimeVO getServerTime(String siteCode, String historyId) {
        ServerTimeVO ServerTimeVO = new ServerTimeVO();

        // 获取套餐
        SeoPlanSubscription history = seoPlanSubscriptionService.getById(historyId);
        if (history == null) {
            return ServerTimeVO;
        }

        // 判断是否自定义套餐结束，如果是自定义套餐，需要获得套餐开启时间
        if (history.getServiceStatus() == 1) {
            ServerTimeVO.setPlanServiceEndStatus(1);
            ServerTimeVO.setPlanStartTime(history.getPlanStartTime());
        }

        // 达成时间
        Date achieveTime = history.getKeywordsAchieveTime();
        if (achieveTime != null) {
            ServerTimeVO.setReachStandardTime(
                    new SimpleDateFormat(DateUtil.DATE_PATTERN).format(achieveTime));
        }

        // 达成天数
        int reachStandardDays = 0;
        if (achieveTime != null) {
            QueryWrapper<HistoryReachStandardSite> historyReachStandardSiteQueryWrapper =
                    new QueryWrapper<>();
            historyReachStandardSiteQueryWrapper.eq("site_code", siteCode);
            List<HistoryReachStandardSite> historyReachStandardSiteList =
                    historyReachStandardSiteService.list(historyReachStandardSiteQueryWrapper);
            if (ListUtil.isEmpty(historyReachStandardSiteList)) {
                reachStandardDays = (int) DateUtil.diffDays(achieveTime, new Date());
            } else {
                HistoryReachStandardSite historyReachStandardSite =
                        historyReachStandardSiteList.get(0);
                int newDays =
                        (int)
                                DateUtil.diffDays(
                                        historyReachStandardSite.getCreateTime(), new Date());
                reachStandardDays = historyReachStandardSite.getReachStandardDays() + newDays;
            }
        }
        ServerTimeVO.setReachStandardDays(reachStandardDays);

        // 剩余服务天数，0指定服务时长，1指定服务结束日期
        int remainServerDays;
        if (history.getServiceStatus() == 0) {
            int serverMonth = history.getServiceMonth();
            int serverDay = (serverMonth * 30) + (serverMonth / 2);
            int giveDay = history.getGiveDay();
            int compensateDay = history.getCompensateDay();
            int totalGiveAndCompensateDay = giveDay + compensateDay;
            log.info(
                    "getServerTime -- historyId:{},giveDay:{},compensateDay:{},totalGiveAndCompensateDay:{}",
                    history.getId(),
                    giveDay,
                    compensateDay,
                    totalGiveAndCompensateDay);
            remainServerDays = serverDay + totalGiveAndCompensateDay - reachStandardDays;
            log.info(
                    "getServerTime -- 剩余服务天数:{},服务天数:{},赠、补天数:{},达成天数:{}",
                    remainServerDays,
                    serverDay,
                    totalGiveAndCompensateDay,
                    reachStandardDays);
            if (remainServerDays < 0) {
                remainServerDays = 0;
            }
        } else {
            int giveDay = history.getGiveDay();
            int compensateDay = history.getCompensateDay();
            int totalGiveAndCompensateDay = giveDay + compensateDay;
            log.info(
                    "getServerTime -- historyId:{},giveDay:{},compensateDay:{},totalGiveAndCompensateDay:{}",
                    history.getId(),
                    giveDay,
                    compensateDay,
                    totalGiveAndCompensateDay);
            Date serverEndTime = history.getServiceEndTime();
            log.info(
                    "getServerTime -- 原来的serverEndTime:{}",
                    new SimpleDateFormat(DateUtil.DATE_PATTERN).format(serverEndTime));
            serverEndTime = DateUtil.addDays(serverEndTime, totalGiveAndCompensateDay);
            log.info(
                    "getServerTime -- 增加赠、补天数:{},赠送天数、补偿天数后的serverEndTime:{}",
                    totalGiveAndCompensateDay,
                    new SimpleDateFormat(DateUtil.DATE_PATTERN).format(serverEndTime));
            if (serverEndTime.compareTo(new Date()) <= 0) {
                remainServerDays = 0;
            } else {
                remainServerDays = (int) DateUtil.diffDays(serverEndTime, new Date());
            }
            log.info(
                    "getServerTime -- 剩余服务天数:{},赠、补天数:{},达成天数:{}",
                    remainServerDays,
                    totalGiveAndCompensateDay,
                    reachStandardDays);
        }
        ServerTimeVO.setRemainServerDays(remainServerDays);
        return ServerTimeVO;
    }

    private Map<String, Integer> dealKeywordsRankData(
            String siteCode, String historyId, Integer rankStart, Integer rankEnd) {
        Map<String, Integer> KeywordsRankMap = new HashMap<>();

        // 指定词
        List<String> appointDateList =
                seoKeywordsService.getKeywordsLastSevenDays(siteCode, 1, historyId);
        String appointDatesStr = "";
        if (ListUtil.notEmpty(appointDateList)) {
            for (int i = 0; i < appointDateList.size(); i++) {
                if (i == appointDateList.size() - 1) {
                    appointDatesStr += "'" + appointDateList.get(i) + "'";
                } else {
                    appointDatesStr += "'" + appointDateList.get(i) + "'" + ",";
                }
            }
            appointDatesStr = "(" + appointDatesStr + ")";

            // 指定词
            RankInfoVO appointRankInfo =
                    seoKeywordsMapper.queryKeywordNumByRankRange(
                            siteCode, historyId, rankStart, rankEnd, appointDatesStr, "1");
            if (appointRankInfo != null) {
                KeywordsRankMap.put("appoint", appointRankInfo.getKeywordNum());
            }
        }

        // 长尾词
        List<String> dateList = seoKeywordsService.getKeywordsLastSevenDays(siteCode, 2, historyId);
        String dateStr = "";
        if (ListUtil.notEmpty(dateList)) {
            for (int i = 0; i < dateList.size(); i++) {
                if (i == dateList.size() - 1) {
                    dateStr += "'" + dateList.get(i) + "'";
                } else {
                    dateStr += "'" + dateList.get(i) + "'" + ",";
                }
            }
            dateStr = "(" + dateStr + ")";
            RankInfoVO longRankInfo =
                    seoKeywordsMapper.queryKeywordNumByRankRange(
                            siteCode, historyId, rankStart, rankEnd, dateStr, "2");
            if (longRankInfo != null) {
                KeywordsRankMap.put("longTail", longRankInfo.getKeywordNum());
            }
        }
        return KeywordsRankMap;
    }

    /**
     * 给列表中的关键字增加排名信息
     * 根据关键词类型
     * @param keywordList 关键词列表
     *
     */
    @Override
    public void addRankInfoByIntervalTimeByType(List<SeoKeywords> keywordList, Integer keywordType, String siteCode, String historyId, List<String> dateList) throws Exception {
        if (ListUtil.isEmpty(keywordList)) {
            return;
        }

        String datesStr = "";
        if(ListUtil.isEmpty(dateList)){
            return;
        }else{
            for(int i = 0; i < dateList.size(); i++){
                if(i == dateList.size() - 1){
                    datesStr +=  "'" + dateList.get(i) + "'";
                }else{
                    datesStr +=  "'" + dateList.get(i) + "'" + ",";
                }
            }
            datesStr = "(" + datesStr + ")";
        }
        List<Integer> keywordIds = keywordList.stream().map(SeoKeywords::getId).collect(Collectors.toList());
        List<SeoKeywordsSerp> serpList = seoKeywordsSerpService.list(new LambdaQueryWrapper<SeoKeywordsSerp>()
                .in(SeoKeywordsSerp::getKeywordsId, keywordIds)
                .in(SeoKeywordsSerp::getSeDate, dateList));
        List<SeoKeywordsSerp> noRepeatSerpList = new ArrayList<>();
        Map<String, Integer> serpMap = new HashMap<>();
        if(ListUtil.notEmpty(serpList)){
            for (SeoKeywordsSerp seoKeywordsSerp : serpList) {
                seoKeywordsSerp.setKey(seoKeywordsSerp.getKeywordsId() + "_" + seoKeywordsSerp.getSeDate());
                if(ListUtil.notEmpty(noRepeatSerpList)){
                    List<SeoKeywordsSerp> have = noRepeatSerpList.stream().filter(o -> o.getKey().equals(seoKeywordsSerp.getKey())).collect(Collectors.toList());
                    if(ListUtil.isEmpty(have)){
                        serpMap.put(seoKeywordsSerp.getKey(),seoKeywordsSerp.getRankAbsolute());
                        noRepeatSerpList.add(seoKeywordsSerp);
                    }else{
                        log.info("重复的数据：" + seoKeywordsSerp.getKey());
                    }
                }else {
                    serpMap.put(seoKeywordsSerp.getKey(),seoKeywordsSerp.getRankAbsolute());
                    noRepeatSerpList.add(seoKeywordsSerp);
                }
            }
        }

        for (SeoKeywords keyword : keywordList) {
            Map<String, String> rankInfo = new LinkedHashMap<>();
            for (String date : dateList) {

                // 从Map中获取那天的排名信息
                Integer rankAbsolute = serpMap.get(keyword.getId() + "_" + date);

                if(rankAbsolute == null){
                    rankAbsolute = 0;
                }
                rankInfo.put(date, rankAbsolute + "");
            }

            keyword.setRankInfo(rankInfo);
            keyword.setIpAddress("223.112.18.226");

            keyword.setDateList(dateList);
            if(StringUtil.isEmpty(keyword.getSearchUrl())){
                keyword.setSearchUrl("https://www.google.com/");
            }
        }
    }

    /**
     * 自适应宽度(中文支持)
     * @param sheet
     * @param size 因为for循环从0开始，size值为 列数-1
     */
    @Override
    public void setSizeColumn(Sheet sheet, int size) {
        for (int columnNum = 0; columnNum <= size; columnNum++) {
            int columnWidth = sheet.getColumnWidth(columnNum) / 256;
            for (int rowNum = 0; rowNum <= sheet.getLastRowNum(); rowNum++) {
                Row currentRow;
                //当前行未被使用过
                if (sheet.getRow(rowNum) == null) {
                    currentRow = sheet.createRow(rowNum);
                } else {
                    currentRow = sheet.getRow(rowNum);
                }

                if (currentRow.getCell(columnNum) != null) {
                    Cell currentCell = currentRow.getCell(columnNum);
                    if (currentCell.getCellType() == CellType.STRING) {
                        int length = currentCell.getStringCellValue().getBytes().length;
                        if (columnWidth < length) {
                            columnWidth = length;
                        }
                    }
                }
            }
            sheet.setColumnWidth(columnNum, columnWidth * 256);
        }
    }
}
